#include "darwin_leg_kinematics.h"
#include "darwin_robot_exceptions.h"

#include <iostream>

#define     PI    3.14159

std::string kin_file="../src/xml/right_leg.xml";

int main(int argc, char *argv[])
{
  double i,j,k,l,m,n,error;
  int num_total=0,num_sol=0;
  std::vector<double> joints_in(6),pos_in,rot_in;
  std::vector<double> joints_out(6),pos_out,rot_out;

  try{
    CDarwinLegKinematics kin;
    kin.load_chain(kin_file);

    joints_in[0]=atof(argv[1]);
    joints_in[1]=atof(argv[2]);
    joints_in[2]=atof(argv[3]);
    joints_in[3]=atof(argv[4]);
    joints_in[4]=atof(argv[5]);
    joints_in[5]=atof(argv[6]);
    kin.get_forward_kinematics(joints_in,pos_out,rot_out);
    std::cout << "position:" << pos_out[0] << "," << pos_out[1] << "," << pos_out[2] << std::endl;
    std::cout << "orientation:" << rot_out[0] << "," << rot_out[1] << "," << rot_out[2] << std::endl;
    pos_in=pos_out;
    rot_in=rot_out;
    kin.get_inverse_kinematics(pos_in,rot_in,joints_out);
    std::cout << "joints: " << joints_out[0] << "," << joints_out[1] << "," << joints_out[2] << "," << joints_out[3] << "," << joints_out[4] << "," << joints_out[5] << "," << std::endl;
    for(i=-PI/4;i<PI/4;i+=0.2)
    {
      for(j=-PI/4;j<PI/4;j+=0.2)
      {
        for(k=-PI/4;k<PI/4;k+=0.2)
        {
          for(l=0;l<PI/4;l+=0.2)
          {
            for(m=-PI/4;m<PI/4;m+=0.2)
            {
              for(n=-PI/4;n<PI/4;n+=0.2)
              {
                num_total++;
                joints_in[0]=i;
                joints_in[1]=j;
                joints_in[2]=k;
                joints_in[3]=l;
                joints_in[4]=m;
                joints_in[5]=n;
                try{
                  std::cout << i << "," << j << "," << k << "," << l << "," << m << "," << n << std::endl;
                  kin.get_forward_kinematics(joints_in,pos_out,rot_out);
                  pos_in=pos_out;
                  rot_in=rot_out;
                  kin.get_inverse_kinematics(pos_in,rot_in,joints_out);
                  error=0.0;
                  error+=pow(i-joints_out[0],2.0);
                  error+=pow(j-joints_out[1],2.0);
                  error+=pow(k-joints_out[2],2.0);
                  error+=pow(l-joints_out[3],2.0);
                  error+=pow(m-joints_out[4],2.0);
                  error+=pow(n-joints_out[5],2.0);
                  error=sqrt(error);
                  std::cout << joints_out[0] << "," << joints_out[1] << "," << joints_out[2] << "," << joints_out[3] << "," << joints_out[4] << "," << joints_out[5] << std::endl;
                  if(error>0.1)
                    std::cout << "Error: " << error << std::endl;
                  else
                    num_sol++;
                }catch(CException &e){
                  std::cout << e.what() << std::endl;
                }
              }
            } 
          }
        }
      }
    }
    std::cout << "Found " << num_sol << " solutions of " << num_total << " attempts" << std::endl;

  }catch(CException &e){
    std::cout << e.what() << std::endl;
  }
}
