
/* DRIVER of the DARWIN ROBOT firmware
 * 
 * Test with the robot darwin with smart charger connected to dynamixel v2, baudrate 1M
 * 
 * In terminal:
 * 	Default period of the smart charger (read smart charger data period)
 *	Smart charger detected, not detected or detected and enabled 
 *	Start motion manager (if smart charger detected and enabled, it will execute the read operations)
 * 	Battery status: connected, disconnected or connected and charging
 * 	Shows average time to full and to empty in min (65535 it means -1)
 * 
 NOTES: First download darwin_stm32_fw with cm730_fw main code (MakeFile)
 * 
 * 
 */



#include "darwin_robot.h"
#include "darwin_robot_exceptions.h"

#include <iostream>

//thread
std::string robot_device="A603LOBS";
//std::string robot_device="A4008atn";

int main(int argc, char *argv[])
{
  //unsigned short int period;
  double period;
  int control, status;
  TChargerData charger_data;
  double current;

  try{
    CDarwinRobot darwin("Darwin",robot_device,1000000,0x02);
    std::cout << "found darwin controller" << std::endl;

  darwin.gpio_clear_led(LED_TX);
  darwin.gpio_clear_led(LED_4);
  darwin.gpio_clear_led(LED_3);
  
  std::cout << "Holaaaaa! " << std::endl;
  
//Read/write smart charger period    
    std::cout << "Default smart charger period: " << darwin.smart_charger_get_period() << " seg" << std::endl;
    std::cout << "Changing smart charger period to 1500ms..." << std::endl;
    darwin.smart_charger_set_period(1.5);
    std::cout << "New smart charger period: " << darwin.smart_charger_get_period() << " seg" << std::endl;
//Write invalid period value  
/*    std::cout << "Changing smart charger period to 1000ms" << std::endl;
    darwin.smart_charger_set_period(1.00);
    std::cout << "smart charger period (s): " << darwin.smart_charger_get_period() << std::endl;
*/   

//DETECTING AND ENABLING SMART CHARGER
/*    if(darwin.smart_charger_is_detected())
      std::cout << "smart charger detected" << std::endl;
*/    
    control=darwin.smart_charger_read_control();
    if(control==1)
      std::cout << "smart charger detected" << std::endl;
    else if(control==0)
      std::cout << "smart charger not detected" << std::endl;
    
    std::cout << "Enabling smart charger module..." << std::endl;
    darwin.smart_charger_enable();
    control=darwin.smart_charger_read_control();
    if(control==3)
      std::cout << "smart charger detected and enabled" << std::endl;

//START MOTION MANAGER
     darwin.mm_start();
    if(darwin.mm_is_running())
    std::cout << "Motion manager is running" << std::endl;
    
//WRITE EEPROM
    double max_current=1.0;
    double min_current=0.255;
    double limit_current = 0.512; //(A)
      std::cout << "Setting limit current to 0.512 A..." << std::endl;
  //  darwin.smart_charger_range_current(min_current, max_current);
    darwin.smart_charger_set_limit_current(limit_current);
   
    //current=darwin.smart_charger_get_limit_current();
    std::cout << "LIMIT CURRENT: " << darwin.smart_charger_get_limit_current() << "A" << std::endl;
    
    
//READ SMART CHARGER DATA
    
  while(1){
    charger_data=darwin.smart_charger_get_data();
    if(charger_data.bat_status==0)
      std::cout << "battery status: disconnected " << std::endl;
    else if(charger_data.bat_status==192)
      std::cout << "battery status: connected" << std::endl;
    else if(charger_data.bat_status==128)
      std::cout << "battery status: connected and charging " << std::endl;
    else
      std::cout << "Battery status value: " << charger_data.bat_status << std::endl;
    //y totalmente cargado o descargado???
    
    std::cout << "Avg time to empty: " << charger_data.avg_time_empty << " min" << std::endl;
    std::cout << "Avg time to full: " << charger_data.avg_time_full << " min" << std::endl;
    std::cout << "---------------------- "  << std::endl;
    //std::cout << "Avg time to empty: " << darwin.smart_charger_read_time_empty() << " min" << std::endl;
    
    
    //charger status --> batteries detected, AC detected, etc
    //status=darwin.smart_charger_read_status();
    //std::cout << "charger status: " << status << std::endl;
    //3=solo conector
    //5=solo bateria
    //7=conector + bateria
    

    
  
   // darwin.gpio_toggle_led(LED_RX); //naranja
      
    usleep(500000);
    }    
    
    
    
  }catch(CException &e){
    std::cout << e.what() << std::endl;
  }
}
